require 'json'
require 'yaml'

module GSM
  def self.create(game, id, port, opt)
    # check if game valid
    configs = YAML.load_file("#{__dir__}/games.yml")
    raise "game '#{game}' is not yet supported" unless configs.key? game

    # get server configs
    config = configs[game]
    port ||= config['port_default']
    command = "docker run -e DOCKERGSM_IMAGE #{game} #{config['options']} " \
    "#{config['port_opt'].gsub!('PORT', port.to_s)} " \
    "--name #{id} #{config['image']} #{opt}"

    # create server
    `#{command}`
    raise 'failed to create server' unless $?.success?
  end

  def self.start(id)
    # start server
    `docker start #{id}`
    raise 'failed to start server' unless $?.success?
  end

  def self.console(id)
    # get game type
    data = `docker inspect -f '{{range .Config.Env}}{{println .}}{{end}}' #{id}`
    raise "server #{id} does not exist" unless $?.success?

    # run specific console
    image = data.split(/\n/).find { |e| /DOCKERGSM_IMAGE/ =~ e }[16..-1]
    config = YAML.load_file("#{__dir__}/games.yml")[image]
    puts "INFO: exit the console with #{config['console_esc']}"
    exec config['console'].sub!('ID', id)
  end

  def self.status(id)
    # get inspect data
    data = `docker inspect -f '{{.State.Status}}' #{id}`
    raise "server #{id} does not exist" unless $?.success?

    # output status
    puts "Status of #{id}: #{data}"
  end

  def self.stop(id)
    # stop server
    `docker stop #{id}`
    raise 'failed to stop server' unless $?.success?
  end

  def self.delete(id)
    # delete server
    `docker rm #{id}`
    raise 'failed to delete server' unless $?.success?
  end
end
